package ca.recrown.islandsurvivalcraft.world;

import java.util.HashMap;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;

import org.bukkit.World;
import org.bukkit.plugin.java.JavaPlugin;

import ca.recrown.islandsurvivalcraft.world.generation.IslandBiomeGenerator;
import ca.recrown.islandsurvivalcraft.world.generation.IslandWorldGenerator;

/**
 * Alternates the data used on a per world basis.
 * Uses IslandWorldGenerator as the container for each world.
 */
public class IslandWorldGeneratorAlternator {
    private final HashMap<UUID, IslandWorldGenerator> chunkGenerators;
    private volatile IslandWorldGenerator lastGenerator;
    private volatile UUID lastUUID;
    private final IslandBiomeGenerator islandBiomeGenerator;
    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock(true);
    private final ReadLock readLock = lock.readLock();
    private final WriteLock writeLock = lock.writeLock();
    private final JavaPlugin plugin;

    public IslandWorldGeneratorAlternator(JavaPlugin plugin, IslandBiomeGenerator biomeGenerator) {
        this.chunkGenerators = new HashMap<>();
        this.islandBiomeGenerator = biomeGenerator;
        this.plugin = plugin;
    }

    public IslandWorldGenerator getIslandChunkGeneratorSystem(World world, Random random) {
        if (lastUUID == null || (lastUUID.hashCode() != world.getUID().hashCode() && lastUUID.equals(world.getUID()))) {
            System.out.println("Alternating generator for world: " + world.getName());
            lastUUID = world.getUID();
            readLock.lock();
            try {
                lastGenerator = chunkGenerators.get(lastUUID);
            } finally {
                readLock.unlock();
            }
            if (lastGenerator == null) {
                lastGenerator = new IslandWorldGenerator(plugin, world, islandBiomeGenerator, random);
                writeLock.lock();
                try {
                    chunkGenerators.put(lastUUID, lastGenerator);
                } finally {
                    writeLock.unlock();
                }
            }
        }
        return lastGenerator;
    }
}