package ca.recrown.islandsurvivalcraft;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.ArrayList;
import java.util.HashMap;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestInstance.Lifecycle;

import ca.recrown.islandsurvivalcraft.Types.Point2;

/**
 * Unit test for simple App.
 */
@TestInstance(Lifecycle.PER_CLASS)
public class UtilitiesTest {
    /**
     * Basic hashmap inversion test.
     */
    @Test
    public void testInvertHashMap()
    {
        HashMap<String, Integer> hashMap = new HashMap<>();
        hashMap.put("a", 1);
        hashMap.put("b", 1);
        hashMap.put("c", 2);
        hashMap.put("d", 2);

        HashMap<Integer, ArrayList<String>> res = Utilities.invertHashMap(hashMap);
        ArrayList<String> first = res.get(1);
        ArrayList<String> expectedFirst = new ArrayList<>();
        expectedFirst.add("a");
        expectedFirst.add("b");

        ArrayList<String> second = res.get(2);
        ArrayList<String> expectedSecond = new ArrayList<>();
        expectedSecond.add("c");
        expectedSecond.add("d");
        assertEquals(first, expectedFirst);
        assertEquals(second, expectedSecond);
    }

    @Test
    public void testMagnitudeAddPositive() {
        assertEquals(2, Utilities.addMagnitude(1, 1));
    }
    @Test
    public void testMagnitudeAddNegative() {
        assertEquals(-2, Utilities.addMagnitude(-1, 1));
    }

    @Test
    public void testWorldToChunkCoordinatesNegativePerfect() {
        assertEquals(new Point2(-4, -3), Utilities.worldToChunkCoordinates(-64, -48));
    }

    @Test
    public void testWorldToChunkCoordinatesNegativeOff() {
        assertEquals(new Point2(-4, -3), Utilities.worldToChunkCoordinates(-55, -33));
    }

    @Test
    public void testWorldToChunkCoordinatesPositivePerfect() {
        assertEquals(new Point2(4, 5), Utilities.worldToChunkCoordinates(64, 80));
    }

    @Test
    public void testWorldToChunkCoordinatesPositiveOff() {
        assertEquals(new Point2(4, 5), Utilities.worldToChunkCoordinates(67, 84));
    }

    @Test
    public void testWorldToLocalCoordinatesNegative() {
        assertEquals(new Point2(2, 5), Utilities.worldToLocalChunkCoordinates(-62, -27));
    }

    @Test
    public void testWorldToLocalCoordinatesPositive() {
        assertEquals(new Point2(7, 3), Utilities.worldToLocalChunkCoordinates(39, 83));
    }
}
